@@ -63,8 +63,6 @@ _dbus_write_two (int               fd,
                  int               start2,
                  int               len2);
 
-dbus_bool_t _dbus_open_unix_socket (int              *fd,
-                                    DBusError        *error);
 int _dbus_connect_unix_socket (const char     *path,
                                dbus_bool_t     abstract,
                                DBusError      *error);
@@ -72,6 +70,10 @@ int _dbus_listen_unix_socket  (const char     *path,
                                dbus_bool_t     abstract,
                                DBusError      *error);
 
+int _dbus_connect_exec (const char     *path,
+                        char *const    argv[],
+                        DBusError      *error);
+
 int _dbus_listen_systemd_sockets (int       **fd,
                                  DBusError *error);
 
@@ -130,11 +132,12 @@ void        _dbus_group_info_free     (DBusGroupInfo    *info);
 
 dbus_uid_t    _dbus_getuid (void);
 dbus_uid_t    _dbus_geteuid (void);
-dbus_gid_t    _dbus_getgid (void);
 
 dbus_bool_t _dbus_parse_uid (const DBusString  *uid_str,
                              dbus_uid_t        *uid);
 
+void _dbus_close_all (void);
+
 /** @} */
 
 DBUS_END_DECLS
